#! /usr/bin/python3

import argparse

import sys
import os
import csv
#import ipdb
import testeur
import pandas
import json
import re

# Configuration
default_host = "localhost"
default_user = "ubuntu"

dockers = {}

nbOK = 0
nbNOK = 0

def ok(msg):
    global nbOK
    
    nbOK=nbOK+1
    print ("\033[48;5;100m\033[1m{}\033[0m\t{}".format("OK", msg))

def nok(msg):
    global nbNOK
    
    nbNOK=nbNOK+1
    print ("\033[48;5;196m\033[1m\33[5m{}\033[0m\t{}".format("Not OK", msg))


def get_descript(t, iddocker):
    """
    Récupère une descrition d'un docker dont on a le nom a priori a partir
    de la commande docker inspect
    """
    sortie, err = t.test("sudo docker inspect {}".format(iddocker))
    if len(err) != 0:
        raise ValueError("Le docker {} n'existe pas".format(iddocker))
    descr = {}
    inspect = json.loads("".join(sortie))[0]

    descr['Name'] = inspect['Name']
    descr['HostnamePath'] = inspect['HostnamePath']
    descr['HostsPath'] = inspect['HostsPath']

    descr['Volumes'] = inspect['Mounts']
    descr['Ports'] = inspect['NetworkSettings']['Ports']
    descr['Networks'] = {}
    for net in inspect['NetworkSettings']['Networks'].keys():
        descr['Networks'][net] = {}
        descr['Networks'][net]['IPAddress'] = inspect['NetworkSettings']['Networks'][net]['IPAddress']
        descr['Networks'][net]['Prefix'] = inspect['NetworkSettings']['Networks'][net]['IPPrefixLen']

    descr['Env'] = inspect['Config']['Env']
    descr['ExtraHosts'] = inspect['HostConfig']['ExtraHosts']
    return descr
    
def test_install_docker(t):
    """
    test l'installation de docker, si le docker démon fonctionne et si
    le proxy est bien mis
    """
    sortie, err = t.test("sudo docker ps -a")
    if len(err) != 0:
        print("Il y a une erreur pour la commande docker ps")
        for l in err:
            print("\t\"{}\"".format(l))
        nok("Docker a un problème")
    else:
        ok("Docker semble installé et fonctionnel")

def test_presence_docker(t):
    """
    teste si les dockers apache1, apache2, mysql et nginx existent
    
    effet de bords : stocke la description des dockers utilisés dans un tableau

    """
    sortie, err = t.test("sudo docker ps -a", result="file")
    sort = pandas.read_fwf(sortie,header=0)

    
    print("Dockers trouvés :\n");
    for l in sort['NAMES']:
        print("\t\"{}\"".format(l))

    dat = {
        'docktest' : 'mysql:5.7.23',
        'dockbdd'  : 'mysql:5.7.23',
        'dockpma'  : 'phpmyadmin',
        'docknginx': 'nginx',
        'dockowncloud':'owncloud-perso'}

    dockers = {}
    bon = True
    for nom in dat.keys():
        # le docker est-il dans la liste des noms
        trouv = sort['NAMES'][nom == sort['NAMES']]
        if len(trouv) == 1:
            pos = trouv.index[0]
            ok("Docker \"{}\" trouvé en position {}".format(nom, pos))
            dockers[nom] = get_descript(t, nom)
            if re.match("^{}.*$".format(dat[nom]),sort['IMAGE'][pos]):
                 ok("Le nom de l'image est bien \"{}\"".format(sort['IMAGE'][pos]))
            else:
                 bon = False
                 nok("Le nom de l'image est \"{}\" et pas \"{}\""
                     .format(sort['IMAGE'][pos], dat[nom]))
        elif len(trouv) > 1:
            bon = False
            nok("\"{}\" dockers trouvés".format(len(trouve)))
        else:
            bon = False
            nok("Docker \"{}\" non trouvé".format(nom))
            nok("Donc l'image n'est pas bonne non plus")

    return dockers

def verif_partage(vol, source, dest, moderw):
    print("vérifie le partage de {} sur {} en mode RW {}"
          .format(vol['Source'], vol['Destination'], vol['RW']))
    return ((re.match(source,vol['Source']))
            and (re.match(dest,vol['Destination']))
            and (vol['RW'] == moderw))
    
def test_docktest(docker, t):
    result, err = t.test("sudo cat {}".format(docker['HostnamePath']))
    if (result[0]=='test\n') :
        ok("Ne hostname du docker est bien test")
    else:
        nok("le hostname du docker est {} au lieu de test".format(result[0]))

    if (any(map(lambda x: x=="MYSQL_ROOT_PASSWORD=passbdd", docker['Env']))):
        ok("on trouve la variable d'env pour le password de root")
    else:
        nok("Le mot de passe root ne semble pas bon")

    if (any(map(lambda x:
                verif_partage(x,
                              "/home/ubuntu/docker/datatest",
                              "/var/lib/mysql",
                              True
                ),
                docker['Volumes']))):
        ok("Le partage de données est trouvé")
    else:
        nok("PAs de partage pour les données")


    res, err = t.test('ls /home/ubuntu/docker/datatest/')
    if (any(map(lambda x: x=='MABASE\n', res))):
        ok("On trouve la base MABASE")
    else:
        print('les fichiers du partage "/home/ubuntu/docker/datatest/" sont ')
        for f in res:
            print ("  - {}".format(f))
        nok("On ne trouve la base MABASE")

    res, err = t.test('ls /home/ubuntu/docker/datatest/')
    if (any(map(lambda x: x=='MABASE2\n', res))):
        ok("On trouve la base MABASE2")
    else:
        print('les fichiers du partage "/home/ubuntu/docker/datatest/" sont ')
        for f in res:
            print ("  - {}".format(f))
        nok("On ne trouve la base MABASE2")

    return True

def verif_venv(Venvs, valeur, messageOK, messageNOK):
    try:
        if (any(map(lambda x: re.match(valeur, x), Venvs))):
            ok(messageOK)
            return True
    except:
        print("{} n'est pas iterable".format(Venvs))
    
    nok(messageNOK)
    return False
        
def verif_ip(docker, reseau, ip):
    if reseau in docker['Networks'].keys():
        if (docker['Networks'][reseau]['IPAddress'] == ip):
            ok("{} a bien l'adresse {} dans le réseau {}"
               .format(docker['Name'], ip, reseau)
            )
            return True
        else:
            nok("{} a l'adresse {} au lieu de {} dans le réseau {}"
               .format(docker['Name'],
                       docker['Networks'][reseau]['IPAddress'], ip, reseau))
            return False
    else:
        nok("{} n'a pas d'adresse dans le réseau {}"
            .format(docker['Name'], reseau))
        return False

def test_dockbdd(docker, t):
    result, err = t.test("sudo cat {}".format(docker['HostnamePath']))
    if ((len(result)>0) and (result[0]=='bdd\n')) :
        ok("Ne hostname du docker est bien bdd")
    else:
        nok("le hostname du docker est {} au lieu de bdd".format(result[0]))

    print("Vérification des variables d'environement")
    venvs = [
        ["MYSQL_ROOT_PASSWORD=passbdd", "On trouve le mot de passe root", "Le mot de passe root n'est pas bon"],
        ["MYSQL_USER=owncloud","On trouve l'utilisateur owncloud", "On ne trouve pas l'utilisateur owncloud"],
        ["MYSQL_PASSWORD=passowc","MdP de owncloud", "mauvais MdP pour owncloud"],
        ["MYSQL_DATABASE=owncloud","La base de donnée", "Pas la base de donnée"]
    ]
    for ve in venvs:
        verif_venv(docker["Env"], ve[0], ve[1], ve[2])

    if (any(map(lambda x:
                verif_partage(x,
                              "/home/ubuntu/docker/data",
                              "/var/lib/mysql",
                              True
                ),
                docker['Volumes']))):
        ok("Le partage de données est trouvé")
    else:
        nok("Pas de partage pour les données")


    res, err = t.test('ls /home/ubuntu/docker/data/')
    if (any(map(lambda x: x=='owncloud\n', res))):
        ok("On trouve la base owncloud")
    else:
        print('les fichiers du partage "/home/ubuntu/docker/datatest/" sont ')
        for f in res:
            print ("  - {}".format(f))
        nok("On ne trouve la base owncloud")

    verif_ip(docker, "interne", "172.18.100.10")
        
    return True
    
def test_dockpma(docker, t):

    print("Vérification des variables d'environement")
    venvs = [
        ["PMA_HOST=bdd", "La base est bien configurée", "La base de donnée n'est pas bien configurée le serveur doit être nomé 'bdd'"],
        ["PMA_ABSOLUTE_URI=(http://192.168.[0-9]+.[0-9]+)?/phpMyAdmin/?","L'url d'accès est bonne pour nginx", "L'url d'accès n'est pas bonne pour nginx"],
    ]
    for ve in venvs:
        verif_venv(docker["Env"], ve[0], ve[1], ve[2])

    verif_ip(docker, "interne", "172.18.100.11")
    verif_venv(docker["ExtraHosts"], "bdd:172.18.100.10",
               "La liaison avec la bdd est bonne",
               "La liaison avec la bdd n'est pas bonne"
    )

    res, err = t.test("wget --no-proxy http://172.18.100.11 -O -")
    if (any(map(lambda x: re.search("HTTP request sent, awaiting response... 200 OK",x), err))):
        ok("dockpma semble répondre")
    else:
        nok("dockpma ne semple pas répondre")

    bon = False
    for lig in res:
        m = re.search("PMA_commonParams", lig)
        if (m):
            ok("dockpma semble être un docker phpMyAdmin")
            m1 = re.search('rootPath:"([^"]*)"', lig)
            if (re.match("/?phpMyAdmin/?",m1.group(1))):
                ok("le rootpath est bien {}".format(m1.group(1)))
            else:
                nok("le rootpath est {}".format(m1.group(1)))
            bon=True
    if (not bon):
        nok("dockpma ne semble pas être un docker phpMyAdmin")


def test_docknginx(docker, t):

    verif_ip(docker, "interne", "172.18.100.5")
    verif_venv(docker["ExtraHosts"], "(dock)?pma:172.18.100.11",
               "La liaison avec pma est bonne",
               "La liaison avec pma n'est pas bonne (la machine doit s'appeléer pma ou dockpma"
    )
    if (any(map(lambda x:
                verif_partage(x,
                              "/home/ubuntu/.*nginx\.conf",
                              "/etc/nginx/nginx.conf",
                              False
                ),
                docker['Volumes']))):
        ok("Le partage de configuration est trouvé")
    else:
        nok("Pas partage de configuration")

    if (any(map(lambda x:
                verif_partage(x,
                              "/home/ubuntu/docker/nginx/?",
                              "/www/?",
                              True
                ),
                docker['Volumes']))):
        ok("Le partage de site trouvé")
    else:
        nok("Pas partage de site")

    res, err = t.test("wget --no-proxy http://localhost/phpMyAdmin/ -O -")
    if (any(map(lambda x: re.search("HTTP request sent, awaiting response... 200 OK",x), err))):
        ok("pma semble répondre via nginx semble répondre")
    else:
        nok("dockpma ne semple pas répondre via nginx")

    bon = False
    for lig in res:
        m = re.search("PMA_commonParams", lig)
        if (m):
            ok("dockpma semble être un docker phpMyAdmin")
            m1 = re.search('rootPath:"([^"]*)"', lig)
            if (re.match("/?phpMyAdmin/?",m1.group(1))):
                ok("le rootpath est bien {}".format(m1.group(1)))
            else:
                nok("le rootpath est {}".format(m1.group(1)))
            bon=True
    if (not bon):
        nok("dockpma ne semble pas être un docker phpMyAdmin")


def test_all(server="localhost", user=None, key=None):
    global nbOK
    global nbNOK
    
    nbOK=0
    nbNOK=0
    
    if server == "localhost":
        t = testeur.testeur()
    else:
        t = testeur.testeur_ssh(server, user, key=key)

    #    test_install_docker(t)
    dockers = test_presence_docker(t)
    #verifie_docker(t)

    print("####################################")
    print("## Vérification du docker dockest ##")
    print("####################################")
    if ('docktest' in dockers.keys()):
        test_docktest(dockers['docktest'], t)
    print("####################################")
    print("## Vérification du docker dockbdd ##")
    print("####################################")
    if ('dockbdd' in dockers.keys()):
        test_dockbdd(dockers['dockbdd'], t)
    print("####################################")
    print("## Vérification du docker dockpma ##")
    print("####################################")
    if ('dockpma' in dockers.keys()):
        test_dockpma(dockers['dockpma'], t)
    print("####################################")
    print("## Vérification du docker docknginx ##")
    print("####################################")
    if ('docknginx' in dockers.keys()):
        test_docknginx(dockers['docknginx'], t)
    
    
    t.close()
    return (nbOK,nbNOK)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Passe des scripts de test à distance ou en local")
    parser.add_argument("-s", "--server", type=str, dest="server", default=default_host, help="La machine sur laquelle se connecter")
    parser.add_argument("-u", "--user", type=str, dest="user", default=default_user, help="Le nom de l'utilisateur pour se connecter sur la machine (n'est pas considéré pour la machine locale")
    parser.add_argument("-k", "--key", type=str, dest="key", default=None, help="nom du fichier de clef")
    
    args = parser.parse_args()

    print("##################################")
    print("##################################")
    print("Test du serveur {}".format(args.server))
    print("##################################")
    print("##################################")
    
    nbok, nbnok = test_all(args.server, args.user,args.key)
    print ("## Au final {}/{}".format(nbOK, nbOK+nbNOK))
    
